@@ -1,5 +1,5 @@
 import abc
-from typing import Iterator, NamedTuple, Iterable, Type
+from typing import Iterable, Iterator, NamedTuple, Type
 
 from hcube.api.models.cube import Cube
 from hcube.api.models.query import CubeQuery
@@ -36,3 +36,15 @@ class CubeBackend(abc.ABC):
         """
         Takes a `CubeQuery` instance and removes all matching records
         """
+
+    @abc.abstractmethod
+    def initialize_storage(self, cube: Type[Cube]) -> None:
+        """
+        Initializes the backing storage for `cube` - creates a table, etc.
+        """
+
+    @abc.abstractmethod
+    def drop_storage(self, cube: Type[Cube]) -> None:
+        """
+        Drops the backing storage for `cube` - drops a table, etc.
+        """
