@@ -1,5 +1,5 @@
 import abc
-from typing import Optional, Any
+from typing import Any, Optional
 
 
 class Metric(abc.ABC):
