@@ -1,19 +1,19 @@
-from typing import List, Type, Any, Union
+from typing import Any, List, Type, Union
 
-from hcube.api.exceptions import ConsistencyError, ConfigurationError
-from hcube.api.models.aggregation import Aggregation, AggregationOp, Count
+from hcube.api.exceptions import ConfigurationError, ConsistencyError
+from hcube.api.models.aggregation import Aggregation, AggregationOp, ArrayAgg, Count
 from hcube.api.models.dimensions import Dimension
 from hcube.api.models.filters import (
-    Filter,
-    ListFilter,
-    IsNullFilter,
     ComparisonFilter,
     ComparisonType,
-    NegativeListFilter,
     EqualityFilter,
+    Filter,
+    IsNullFilter,
+    ListFilter,
+    NegativeListFilter,
 )
 from hcube.api.models.metrics import Metric
-from hcube.api.models.ordering import OrderSpec, OrderDirection
+from hcube.api.models.ordering import OrderDirection, OrderSpec
 from hcube.api.models.transforms import Transform
 
 
@@ -51,10 +51,12 @@ class CubeQuery:
             aggreg.name = name
             self.aggregations.append(aggreg)
         for aggreg in self.aggregations:
+            if aggreg.filters:
+                self._resolve_aggregation_filters(aggreg)
             if aggreg.metric:
                 # resolve the metric when it was given as a string
                 aggreg.metric = self._resolve_metric(aggreg.metric)
-            elif isinstance(aggreg, Count) and aggreg.distinct:
+            elif isinstance(aggreg, (Count, ArrayAgg)) and aggreg.distinct:
                 aggreg.distinct = self._resolve_dim(aggreg.distinct)
             else:
                 if not aggreg.allow_metric_none:
@@ -142,13 +144,13 @@ class CubeQuery:
     def _resolve_shorthand(self, spec: str, value: Any):
         parts = spec.split("__")
         if len(parts) == 1:
-            return EqualityFilter(dimension=self.cube.dimension_by_name(parts[0]), value=value)
+            return EqualityFilter(dimension=self._resolve_dim(parts[0]), value=value)
         if len(parts) != 2:
             raise ValueError("filter spec must be in format `name`__`filter`")
         dim_name, shorthand = parts
         fltr_cls = self.FILTER_SHORTHANDS.get(shorthand)
         if fltr_cls:
-            return fltr_cls(self.cube.dimension_by_name(dim_name), value)
+            return fltr_cls(self._resolve_dim(dim_name), value)
         if shorthand in ("gt", "gte", "lt", "lte"):
             return ComparisonFilter(
                 dimension=self.cube.dimension_by_name(dim_name),
@@ -157,10 +159,13 @@ class CubeQuery:
             )
         raise ValueError(f"unsupported filter `{shorthand}`")
 
-    def _resolve_dim(self, dim: Union[str, Dimension]) -> Dimension:
+    def _resolve_dim(self, dim: Union[str, Dimension]) -> Union[Dimension, Transform]:
         if isinstance(dim, Dimension):
             return self._check_own_dimension(dim)
         else:
+            for transform in self.transforms:
+                if transform.name == dim:
+                    return transform
             return self.cube.dimension_by_name(dim)
 
     def _resolve_metric(self, metric: Union[str, Metric]) -> Metric:
@@ -169,6 +174,16 @@ class CubeQuery:
         else:
             return self.cube.metric_by_name(metric)
 
+    def _resolve_aggregation_filters(self, aggreg: Aggregation):
+        new_filters = []
+        for i, fltr in enumerate(aggreg.filters):
+            if isinstance(fltr, dict):
+                for key, value in fltr.items():
+                    new_filters.append(self._resolve_shorthand(key, value))
+            else:
+                new_filters.append(fltr)
+        aggreg.filters = new_filters
+
     def _check_own_dimension(self, dim: Dimension) -> Dimension:
         if dim not in self.cube._dimensions.values():
             raise ConsistencyError(f'Dimension "{dim}" is not associated with cube: {self.cube}')
