@@ -1,17 +1,17 @@
 from collections import namedtuple
-from typing import Iterator, NamedTuple, Iterable, Type
+from typing import Iterable, Iterator, NamedTuple, Type
 
 from hcube.api.backend import CubeBackend
-from hcube.api.models.aggregation import AggregationOp, Count
+from hcube.api.models.aggregation import AggregationOp, ArrayAgg, Count
 from hcube.api.models.cube import Cube
 from hcube.api.models.filters import (
-    Filter,
-    ListFilter,
-    IsNullFilter,
     ComparisonFilter,
     ComparisonType,
-    NegativeListFilter,
     EqualityFilter,
+    Filter,
+    IsNullFilter,
+    ListFilter,
+    NegativeListFilter,
 )
 from hcube.api.models.ordering import OrderDirection, OrderSpec
 from hcube.api.models.query import CubeQuery
@@ -30,6 +30,12 @@ class NaiveCubeBackend(CubeBackend):
         self.values = []
         self._aggreg_state = {}
 
+    def initialize_storage(self, cube: Type[Cube]) -> None:
+        pass
+
+    def drop_storage(self, cube: Type[Cube]) -> None:
+        pass
+
     def store_records(self, cube: Type[Cube], records: Iterable[NamedTuple]):
         self.values = list(cube.cleanup_records(records))
 
@@ -51,7 +57,13 @@ class NaiveCubeBackend(CubeBackend):
                 if isinstance(agg, Count) and agg.distinct:
                     value = getattr(record, agg.distinct.name)
                     result[key][agg.name] = self._count_distinct(agg.name, value)
+                elif isinstance(agg, ArrayAgg) and agg.distinct:
+                    value = getattr(record, agg.distinct.name)
+                    result[key][agg.name] = self._array_agg_distinct(agg.name, value)
                 else:
+                    if agg.filters:
+                        if not all(self._row_passes_filter(record, fltr) for fltr in agg.filters):
+                            continue
                     value = getattr(record, agg.metric.name) if agg.metric else None
                     result[key][agg.name] = self._aggregate(agg.op, result[key][agg.name], value)
             for transform in query.transforms:
@@ -134,6 +146,10 @@ class NaiveCubeBackend(CubeBackend):
             return min(base, value) if base is not None else value
         elif op == AggregationOp.MAX:
             return max(base, value) if base is not None else value
+        elif op == AggregationOp.ARRAY:
+            return base + [value]
+        else:
+            raise ValueError(f"unsupported aggregation {op}")
 
     def _count_distinct(self, name, value):
         if name not in self._aggreg_state:
@@ -141,10 +157,18 @@ class NaiveCubeBackend(CubeBackend):
         self._aggreg_state[name].add(value)
         return len(self._aggreg_state[name])
 
+    def _array_agg_distinct(self, name, value):
+        if name not in self._aggreg_state:
+            self._aggreg_state[name] = set()
+        self._aggreg_state[name].add(value)
+        return list(self._aggreg_state[name])
+
     @classmethod
     def _base_value(cls, op: AggregationOp):
         if op in (AggregationOp.COUNT, AggregationOp.SUM):
             return 0
+        if op == AggregationOp.ARRAY:
+            return []
         return None
 
     @classmethod
