@@ -1,30 +1,30 @@
 import logging
 from collections import namedtuple
-from typing import Type, Iterable, NamedTuple, Union, Iterator, Optional
+from typing import Iterable, Iterator, NamedTuple, Optional, Type, Union
 
 import psycopg2
 from decouple import config
 
 from hcube.api.backend import CubeBackend
 from hcube.api.exceptions import ConfigurationError
-from hcube.api.models.aggregation import AggregationOp, Count
+from hcube.api.models.aggregation import AggregationOp, ArrayAgg, Count
 from hcube.api.models.cube import Cube
 from hcube.api.models.dimensions import (
+    DateDimension,
+    DateTimeDimension,
     Dimension,
     IntDimension,
     StringDimension,
-    DateDimension,
-    DateTimeDimension,
 )
 from hcube.api.models.filters import (
-    Filter,
-    ListFilter,
-    IsNullFilter,
     ComparisonFilter,
-    NegativeListFilter,
     EqualityFilter,
+    Filter,
+    IsNullFilter,
+    ListFilter,
+    NegativeListFilter,
 )
-from hcube.api.models.metrics import FloatMetric, Metric, IntMetric
+from hcube.api.models.metrics import FloatMetric, IntMetric, Metric
 from hcube.api.models.query import CubeQuery
 from hcube.settings import GlobalSettings
 
@@ -78,8 +78,13 @@ class PostgresCubeBackend(CubeBackend):
         self.connection = psycopg2.connect(database=database, **client_attrs)
         self.schema = schema
 
-    def store_records(self, cube: Type[Cube], records: Iterable[NamedTuple]):
+    def initialize_storage(self, cube: Type[Cube]) -> None:
         self._init_table(cube)
+
+    def drop_storage(self, cube: Type[Cube]) -> None:
+        self._execute(f"DROP TABLE {self.schema}.{self.cube_to_table_name(cube)};")
+
+    def store_records(self, cube: Type[Cube], records: Iterable[NamedTuple]):
         clean_records = cube.cleanup_records(records)
         values_part = ", ".join(f"%({field})s" for field in cube.record_type()._fields)
         self._execute(
@@ -113,7 +118,10 @@ class PostgresCubeBackend(CubeBackend):
         query_parts = {}
         # we run it just for the parts and params
         _text, params, _fields = self._prepare_db_query(query, parts=query_parts)
-        text = "DELETE FROM {table} WHERE {where}".format(**query_parts)
+        if where := query_parts["where"]:
+            text = f"DELETE FROM {query_parts['table']} WHERE {where}"
+        else:
+            text = f"DELETE FROM {query_parts['table']}"
         self._execute(text, params)
 
     def _get_cursor(self):
@@ -138,6 +146,7 @@ class PostgresCubeBackend(CubeBackend):
         returns the query text, parameters to be added during execution and a list of parameter
         names that are expected in the result
         """
+        params = {}
         if query.groups or query.aggregations:
             fields = [grp.name for grp in query.groups]
             select_parts = [*fields]
@@ -146,12 +155,24 @@ class PostgresCubeBackend(CubeBackend):
                 inside = "*"
                 if agg.metric:
                     inside = agg.metric.name
-                elif isinstance(agg, Count) and agg.distinct:
+                elif isinstance(agg, (Count, ArrayAgg)) and agg.distinct:
                     inside = f"DISTINCT {agg.distinct.name}"
+                # filters
+                filter_parts = []
+                filter_text = ""
+                for filtr in agg.filters:
+                    text, agg_params = self._ch_filter(filtr)
+                    filter_parts.append(text)
+                    params.update(agg_params)
+                if filter_parts:
+                    filter_text = "FILTER (WHERE " + " AND ".join(filter_parts) + ")"
+                # coalescing
                 if GlobalSettings.aggregates_zero_for_empty_data:
-                    select_parts.append(f"COALESCE({agg_name}({inside}), 0) AS {agg.name}")
+                    select_parts.append(
+                        f"COALESCE({agg_name}({inside}) {filter_text}, 0) AS {agg.name}"
+                    )
                 else:
-                    select_parts.append(f"{agg_name}({inside}) AS {agg.name}")
+                    select_parts.append(f"{agg_name}({inside}) {filter_text} AS {agg.name}")
                 fields.append(agg.name)
         else:
             fields = [dim.name for dim in query.cube._dimensions.values()] + [
@@ -163,7 +184,6 @@ class PostgresCubeBackend(CubeBackend):
         table = f"{self.schema}.{self.cube_to_table_name(query.cube)}"
         order_by = ", ".join(f"{ob.dimension.name} {ob.direction.name}" for ob in query.orderings)
         where_parts = []
-        params = {}
         for fltr in query.filters:
             filter_text, filter_params = self._ch_filter(fltr)
             where_parts.append(filter_text)
@@ -216,9 +236,12 @@ class PostgresCubeBackend(CubeBackend):
     def _agg_name(self, agg: AggregationOp):
         if agg in (AggregationOp.SUM, AggregationOp.COUNT, AggregationOp.MAX, AggregationOp.MIN):
             return agg.name
+        if agg == AggregationOp.ARRAY:
+            return "ARRAY_AGG"
         raise ValueError(f"Unsupported aggregation {agg}")
 
-    def cube_to_table_name(self, cube: Type[Cube]):
+    @staticmethod
+    def cube_to_table_name(cube: Type[Cube]):
         return cube.__name__.lower()
 
     def _init_table(self, cube: Type[Cube]):
